/*
 * Copyright, OpenVision Technologies, Inc., 1996, All Rights Reserved
 *
 * WARNING:  Retrieving the OpenVision Kerberos Administration system
 * source code, as described below, indicates your acceptance of the
 * following terms.  If you do not agree to the following terms, do not
 * retrieve the OpenVision Kerberos administration system.
 *
 * You may freely use and distribute the Source Code and Object Code
 * compiled from it, but this Source Code is provided to you "AS IS"
 * EXCLUSIVE OF ANY WARRANTY, INCLUDING, WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR
 * ANY OTHER WARRANTY, WHETHER EXPRESS OR IMPLIED.  IN NO EVENT WILL
 * OPENVISION HAVE ANY LIABILITY FOR ANY LOST PROFITS, LOSS OF DATA OR
 * COSTS OF PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, OR FOR ANY
 * SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, THOSE RESULTING FROM THE
 * USE OF THE SOURCE CODE, OR THE FAILURE OF THE SOURCE CODE TO PERFORM,
 * OR FOR ANY OTHER REASON.
 *
 * OpenVision retains all rights, title, and interest in the donated
 * Source Code.  With respect to OpenVision's copyrights in the donated
 * Source Code, OpenVision also retains rights to derivative works of
 * the Source Code whether created by OpenVision or a third party.
 *
 * OpenVision Technologies, Inc. has donated this Kerberos
 * Administration system to MIT for inclusion in the standard Kerberos 5
 * distribution. This donation underscores our commitment to continuing
 * Kerberos technology development and our gratitude for the valuable
 * work which has been performed by MIT and the Kerberos community.
 */

/*
 * $Id: util_set.c,v 1.1 1996/04/30 23:03:38 raeburn Exp $
 */

#include "gssapiP_generic.h"

struct _g_set {
   void *key;
   void *value;
   struct _g_set *next;
};

int g_set_init(g_set *s)
{
   *s = NULL;

   return(0);
}

int g_set_destroy(g_set *s)
{
   g_set next;

   while (*s) {
      next = (*s)->next;
      free(*s);
      *s = next;
   }

   return(0);
}

int g_set_entry_add(g_set *s, void *key, void *value)
{
   g_set first;

   if ((first = (struct _g_set *) malloc(sizeof(struct _g_set))) == NULL)
      return(ENOMEM);

   first->key = key;
   first->value = value;
   first->next = *s;

   *s = first;

   return(0);
}

int g_set_entry_delete(g_set *s, void *key)
{
   g_set *p;

   for (p=s; *p; p = &((*p)->next)) {
      if ((*p)->key == key) {
	 g_set next = (*p)->next;
	 free(*p);
	 *p = next;

	 return(0);
      }
   }

   return(-1);
}

int g_set_entry_get(g_set *s, void *key, void **value)
{
   g_set p;

   for (p = *s; p; p = p->next) {
      if (p->key == key) {
	 *value = p->value;

	 return(0);
      }
   }

   *value = NULL;

   return(-1);
}
